#include <Adafruit_NeoPixel.h>
#include "SPI.h"
#include "NRFLite.h"

const static uint8_t RADIO_ID = 0;             // ID dieses Funkteilnehmers
const static uint8_t PIN_RADIO_CE = 9;         // CE-Pin am Arduino
const static uint8_t PIN_RADIO_CSN = 10;       // CSN-Pin am Arduino

struct RadioPacket      // Datenarray zum Versenden per Funkmodul. 
{                       // Unterschiedliche Datentypen sind möglich. Max 32 Byte
    uint8_t abstand;    // Ermittelte Distanz in CM
    //uint32_t Optionale_variable;
};

NRFLite _radio;
RadioPacket _radioData;

#define LEDPIN 4        // Arduino-Pin für Signalleitung zu LEDs

const int leds = 12;    // Anzahl der LEDs
const int buzzer = 5;   // Arduino-Pin für optionalen 5V Piezo-Buzzer
int sicherheit = 10;     // Sicherheitsabstand (wird zum Messwert addiert) in Zentimeter
long aktiveleds = 0;    // Variable für Anzahl der aktiven LEDs

// NeoPixel LEDs definieren und Farbwert festlegen
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(leds, LEDPIN, NEO_GRB + NEO_KHZ800);
int redColor;  
int greenColor;
int blueColor;
int delayval = 10;      // Verzögerungswert in Millisekunden zur zeitlichen Glättung der Anzeige

void setup() {
Serial.begin(115200);   // Serielle Verbindung aktivieren (Datenrate = 115200 Bit pro Sekunde)
// Kann genutzt werden, um im "Seriellen Monitor" in der Arduino IDE über USB Statusmeldungen abzurufen.

// Initialisieren des Funkmoduls und einstellen der Bitrate und des Kanals  (250 kbit pro Sekunde, Kanal 0)
// Sender und Empfänger müssen gleich konfiguriert sein.
  if (!_radio.init(RADIO_ID, PIN_RADIO_CE, PIN_RADIO_CSN, NRFLite::BITRATE250KBPS, 0))
    {
        Serial.print("Empfänger: ");
        Serial.println("Kommunikation mit Funkmodul läuft nicht.");
        Serial.print("Empfänger: ");
        Serial.println("NRF24-Modul überprüfen.");
        while (1); // Programmende bei Fehler
    }

pinMode(buzzer, OUTPUT);  // Arduino-Pin für Buzzer als Ausgang
digitalWrite(buzzer, LOW);
    
pixels.begin();
pixels.show();          // LED-Kommunikation starten und alle LEDs ausschalten
delay(200);  
}

void loop() {

    while (_radio.hasData())
    {
        _radio.readData(&_radioData); // Datenpaket empfangen
        uint8_t msg = _radioData.abstand;   // Wert für Abstand aus Datenpaket
        
        Serial.print("Empfänger: Abstand in cm: "); 
        Serial.println(msg);  
        
        LEDFarbe();   // Funktion Farbe einstellen (siehe unten)
        aktiveleds = leds - ((_radioData.abstand + sicherheit) * 0.01 * leds);   // Anzahl der LEDs berechnen (Range: 100 bis 0 Zentimeter)
        pixels.clear();
  
        for(int i=0;i<=aktiveleds;i++){
        pixels.setPixelColor(i, pixels.Color(redColor, greenColor, blueColor));
        }

        pixels.show();
        Buzzer();
        delay(delayval);
    }
}

void LEDFarbe(){
// Farbwert der LEDs festlegen.
// 50 bis 100 cm -> Grün
// 30 bis 49 cm -> Gelb
// 0 bis 29 cm -> Rot
switch (_radioData.abstand + sicherheit)
{
case 50 ... 100:
  redColor = 0;
  greenColor = 20;
  blueColor = 0;
  break;
case 30 ... 49:
  redColor = 20;
  greenColor = 20;
  blueColor = 0;
  break;
case 0 ... 29:
  redColor = 20;
  greenColor = 0;
  blueColor = 0;
  break;
default:
  redColor = 20;
  greenColor = 0;
  blueColor = 0;
  break;
}
}

void Buzzer() {
switch (_radioData.abstand + sicherheit)
{
case 50 ... 100:
  noTone(buzzer);
  break;
case 30 ... 49:
  tone(buzzer, 800);
  delay(20);
  break;
case 0 ... 29:
  tone(buzzer, 1000);
  delay(20);
  break;
default:
  noTone(buzzer);
  break;
}
}
////////////////////////
